"""
Command-Line Interface for Collibra Connector.

This module provides a CLI tool for interacting with Collibra
directly from the terminal without writing code.

Installation:
    pip install collibra-connector[cli]

Usage:
    collibra-sdk --help
    collibra-sdk search "Business Term" --format json
    collibra-sdk export-domain --id "uuid" --output report.csv
    collibra-sdk get-asset --id "uuid"
"""
from __future__ import annotations

import csv
import json
import os
import sys
from typing import Any, Dict, List, Optional

try:
    import click
    CLICK_AVAILABLE = True
except ImportError:
    CLICK_AVAILABLE = False

if CLICK_AVAILABLE:
    from click import Context, echo, style, secho


def get_connector():
    """Create connector from environment variables."""
    from .connector import CollibraConnector

    api = os.environ.get("COLLIBRA_URL")
    username = os.environ.get("COLLIBRA_USERNAME")
    password = os.environ.get("COLLIBRA_PASSWORD")

    if not all([api, username, password]):
        raise click.ClickException(
            "Missing environment variables. Set:\n"
            "  COLLIBRA_URL=https://your-instance.collibra.com\n"
            "  COLLIBRA_USERNAME=your-username\n"
            "  COLLIBRA_PASSWORD=your-password"
        )

    return CollibraConnector(api=api, username=username, password=password)


def format_output(data: Any, fmt: str) -> str:
    """Format output data based on format type."""
    if fmt == "json":
        return json.dumps(data, indent=2, default=str)
    elif fmt == "table":
        if isinstance(data, list) and data:
            # Simple table format
            headers = list(data[0].keys())
            lines = ["\t".join(headers)]
            for item in data:
                lines.append("\t".join(str(item.get(h, "")) for h in headers))
            return "\n".join(lines)
        return str(data)
    elif fmt == "csv":
        if isinstance(data, list) and data:
            import io
            output = io.StringIO()
            writer = csv.DictWriter(output, fieldnames=data[0].keys())
            writer.writeheader()
            writer.writerows(data)
            return output.getvalue()
        return str(data)
    else:
        return str(data)


if CLICK_AVAILABLE:
    @click.group()
    @click.version_option(version="1.1.0", prog_name="collibra-sdk")
    @click.pass_context
    def cli(ctx: Context) -> None:
        """
        Collibra SDK - Command line interface for Collibra Data Governance.

        Configure connection using environment variables:

          export COLLIBRA_URL=https://your-instance.collibra.com

          export COLLIBRA_USERNAME=your-username

          export COLLIBRA_PASSWORD=your-password
        """
        ctx.ensure_object(dict)

    # ==========================================================================
    # Connection Commands
    # ==========================================================================

    @cli.command("test")
    def test_connection() -> None:
        """Test the connection to Collibra."""
        try:
            conn = get_connector()
            if conn.test_connection():
                secho("Connection successful!", fg="green")
            else:
                secho("Connection failed.", fg="red")
                sys.exit(1)
        except Exception as e:
            secho(f"Error: {e}", fg="red")
            sys.exit(1)

    @cli.command("info")
    def show_info() -> None:
        """Show connection and version information."""
        try:
            conn = get_connector()
            echo(f"Collibra URL: {conn.base_url}")
            echo(f"SDK Version: {conn.get_version()}")

            if conn.test_connection():
                secho("Status: Connected", fg="green")
            else:
                secho("Status: Not Connected", fg="red")
        except Exception as e:
            secho(f"Error: {e}", fg="red")
            sys.exit(1)

    # ==========================================================================
    # Search Commands
    # ==========================================================================

    @cli.command("search")
    @click.argument("query")
    @click.option("--type", "-t", "asset_type", help="Filter by asset type name")
    @click.option("--domain", "-d", "domain_id", help="Filter by domain ID")
    @click.option("--community", "-c", "community_id", help="Filter by community ID")
    @click.option("--limit", "-l", default=10, help="Maximum results (default: 10)")
    @click.option("--format", "-f", "fmt", default="table",
                  type=click.Choice(["json", "table", "csv"]),
                  help="Output format")
    def search_assets(
        query: str,
        asset_type: Optional[str],
        domain_id: Optional[str],
        community_id: Optional[str],
        limit: int,
        fmt: str
    ) -> None:
        """
        Search for assets in Collibra.

        Examples:

          collibra-sdk search "Customer"

          collibra-sdk search "Product*" --limit 50 --format json

          collibra-sdk search "Order" --type "Business Term"
        """
        try:
            conn = get_connector()

            # Prepare filters
            type_ids = None
            if asset_type:
                # Look up type ID by name
                types = conn.metadata.get_asset_types(name=asset_type, limit=1)
                if types.get("results"):
                    type_ids = [types["results"][0]["id"]]
                else:
                    secho(f"Asset type not found: {asset_type}", fg="yellow")

            domain_ids = [domain_id] if domain_id else None
            community_ids = [community_id] if community_id else None

            # Execute search
            result = conn.search.find_assets(
                query=query,
                limit=limit,
                type_ids=type_ids,
                domain_ids=domain_ids,
                community_ids=community_ids
            )

            # Format results
            items = []
            for item in result.get("results", []):
                resource = item.get("resource", {})
                items.append({
                    "id": resource.get("id"),
                    "name": resource.get("name") or resource.get("displayName"),
                    "type": resource.get("resourceType"),
                    "score": item.get("score", 0)
                })

            if not items:
                echo("No results found.")
                return

            echo(f"Found {result.get('total', 0)} results (showing {len(items)}):\n")
            echo(format_output(items, fmt))

        except Exception as e:
            secho(f"Error: {e}", fg="red")
            sys.exit(1)

    # ==========================================================================
    # Asset Commands
    # ==========================================================================

    @cli.command("get-asset")
    @click.option("--id", "asset_id", required=True, help="Asset UUID")
    @click.option("--format", "-f", "fmt", default="json",
                  type=click.Choice(["json", "table"]),
                  help="Output format")
    @click.option("--full", is_flag=True, help="Include attributes and relations")
    def get_asset(asset_id: str, fmt: str, full: bool) -> None:
        """
        Get details for a specific asset.

        Examples:

          collibra-sdk get-asset --id "abc-123-uuid"

          collibra-sdk get-asset --id "abc-123-uuid" --full
        """
        try:
            conn = get_connector()

            if full:
                result = conn.asset.get_full_profile(asset_id)
                # Flatten for display
                output = {
                    "id": result["asset"].get("id"),
                    "name": result["asset"].get("name"),
                    "display_name": result["asset"].get("displayName"),
                    "type": result["asset"].get("type", {}).get("name"),
                    "status": result["asset"].get("status", {}).get("name"),
                    "domain": result["asset"].get("domain", {}).get("name"),
                    "attributes": result.get("attributes", {}),
                    "relations_outgoing": result["relations"].get("outgoing_count", 0),
                    "relations_incoming": result["relations"].get("incoming_count", 0),
                    "responsibilities": result.get("responsibilities", [])
                }
            else:
                output = conn.asset.get_asset(asset_id)

            echo(format_output(output, fmt))

        except Exception as e:
            secho(f"Error: {e}", fg="red")
            sys.exit(1)

    @cli.command("list-assets")
    @click.option("--domain", "-d", "domain_id", help="Filter by domain ID")
    @click.option("--community", "-c", "community_id", help="Filter by community ID")
    @click.option("--type", "-t", "type_ids", multiple=True, help="Filter by type ID(s)")
    @click.option("--limit", "-l", default=100, help="Maximum results")
    @click.option("--format", "-f", "fmt", default="table",
                  type=click.Choice(["json", "table", "csv"]),
                  help="Output format")
    def list_assets(
        domain_id: Optional[str],
        community_id: Optional[str],
        type_ids: tuple,
        limit: int,
        fmt: str
    ) -> None:
        """
        List assets with optional filters.

        Examples:

          collibra-sdk list-assets --domain "uuid"

          collibra-sdk list-assets --limit 50 --format csv > assets.csv
        """
        try:
            conn = get_connector()

            result = conn.asset.find_assets(
                domain_id=domain_id,
                community_id=community_id,
                asset_type_ids=list(type_ids) if type_ids else None,
                limit=limit
            )

            items = []
            for asset in result.get("results", []):
                items.append({
                    "id": asset.get("id"),
                    "name": asset.get("name"),
                    "type": asset.get("type", {}).get("name"),
                    "status": asset.get("status", {}).get("name"),
                    "domain": asset.get("domain", {}).get("name")
                })

            echo(f"Total: {result.get('total', 0)} assets\n")
            echo(format_output(items, fmt))

        except Exception as e:
            secho(f"Error: {e}", fg="red")
            sys.exit(1)

    # ==========================================================================
    # Export Commands
    # ==========================================================================

    @cli.command("export-domain")
    @click.option("--id", "domain_id", required=True, help="Domain UUID")
    @click.option("--output", "-o", "output_file", required=True,
                  help="Output file path (.csv or .json)")
    @click.option("--include-attributes", is_flag=True, default=True,
                  help="Include asset attributes")
    @click.option("--include-relations", is_flag=True, default=False,
                  help="Include relation summaries")
    @click.option("--limit", "-l", default=1000, help="Maximum assets to export")
    def export_domain(
        domain_id: str,
        output_file: str,
        include_attributes: bool,
        include_relations: bool,
        limit: int
    ) -> None:
        """
        Export all assets in a domain to a file.

        Examples:

          collibra-sdk export-domain --id "uuid" --output assets.csv

          collibra-sdk export-domain --id "uuid" --output data.json --include-relations
        """
        try:
            from .helpers import DataFrameExporter

            conn = get_connector()
            exporter = DataFrameExporter(conn)

            echo(f"Exporting domain {domain_id}...")

            df = exporter.assets_to_dataframe(
                domain_id=domain_id,
                limit=limit,
                include_attributes=include_attributes,
                include_relations=include_relations
            )

            if output_file.endswith(".csv"):
                df.to_csv(output_file, index=False)
            elif output_file.endswith(".json"):
                df.to_json(output_file, orient="records", indent=2)
            elif output_file.endswith(".xlsx"):
                df.to_excel(output_file, index=False)
            else:
                df.to_csv(output_file, index=False)

            secho(f"Exported {len(df)} assets to {output_file}", fg="green")

        except ImportError:
            secho("Pandas is required for export. Install with: pip install pandas", fg="red")
            sys.exit(1)
        except Exception as e:
            secho(f"Error: {e}", fg="red")
            sys.exit(1)

    @cli.command("export-community")
    @click.option("--id", "community_id", required=True, help="Community UUID")
    @click.option("--output", "-o", "output_file", required=True,
                  help="Output file path")
    @click.option("--include-attributes", is_flag=True, default=True)
    @click.option("--limit", "-l", default=1000)
    def export_community(
        community_id: str,
        output_file: str,
        include_attributes: bool,
        limit: int
    ) -> None:
        """
        Export all assets in a community to a file.

        Examples:

          collibra-sdk export-community --id "uuid" --output assets.csv
        """
        try:
            from .helpers import DataFrameExporter

            conn = get_connector()
            exporter = DataFrameExporter(conn)

            echo(f"Exporting community {community_id}...")

            df = exporter.assets_to_dataframe(
                community_id=community_id,
                limit=limit,
                include_attributes=include_attributes
            )

            if output_file.endswith(".csv"):
                df.to_csv(output_file, index=False)
            elif output_file.endswith(".json"):
                df.to_json(output_file, orient="records", indent=2)
            else:
                df.to_csv(output_file, index=False)

            secho(f"Exported {len(df)} assets to {output_file}", fg="green")

        except ImportError:
            secho("Pandas required. Install with: pip install pandas", fg="red")
            sys.exit(1)
        except Exception as e:
            secho(f"Error: {e}", fg="red")
            sys.exit(1)

    # ==========================================================================
    # Community/Domain Commands
    # ==========================================================================

    @cli.command("list-communities")
    @click.option("--format", "-f", "fmt", default="table",
                  type=click.Choice(["json", "table", "csv"]))
    @click.option("--limit", "-l", default=100)
    def list_communities(fmt: str, limit: int) -> None:
        """List all communities."""
        try:
            conn = get_connector()
            result = conn.community.find_communities(limit=limit)

            items = []
            for comm in result.get("results", []):
                items.append({
                    "id": comm.get("id"),
                    "name": comm.get("name"),
                    "description": (comm.get("description") or "")[:50],
                    "parent": comm.get("parent", {}).get("name") if comm.get("parent") else None
                })

            echo(f"Total: {result.get('total', 0)} communities\n")
            echo(format_output(items, fmt))

        except Exception as e:
            secho(f"Error: {e}", fg="red")
            sys.exit(1)

    @cli.command("list-domains")
    @click.option("--community", "-c", "community_id", help="Filter by community ID")
    @click.option("--format", "-f", "fmt", default="table",
                  type=click.Choice(["json", "table", "csv"]))
    @click.option("--limit", "-l", default=100)
    def list_domains(community_id: Optional[str], fmt: str, limit: int) -> None:
        """List all domains."""
        try:
            conn = get_connector()
            result = conn.domain.find_domains(community_id=community_id, limit=limit)

            items = []
            for domain in result.get("results", []):
                items.append({
                    "id": domain.get("id"),
                    "name": domain.get("name"),
                    "type": domain.get("type", {}).get("name"),
                    "community": domain.get("community", {}).get("name")
                })

            echo(f"Total: {result.get('total', 0)} domains\n")
            echo(format_output(items, fmt))

        except Exception as e:
            secho(f"Error: {e}", fg="red")
            sys.exit(1)

    # ==========================================================================
    # Metadata Commands
    # ==========================================================================

    @cli.command("list-asset-types")
    @click.option("--format", "-f", "fmt", default="table",
                  type=click.Choice(["json", "table", "csv"]))
    @click.option("--limit", "-l", default=100)
    def list_asset_types(fmt: str, limit: int) -> None:
        """List all asset types."""
        try:
            conn = get_connector()
            result = conn.metadata.get_asset_types()

            items = []
            for at in result.get("results", [])[:limit]:  # Apply limit manually
                items.append({
                    "id": at.get("id"),
                    "name": at.get("name"),
                    "public_id": at.get("publicId"),
                    "description": (at.get("description") or "")[:40]
                })

            echo(f"Total: {result.get('total', 0)} asset types (showing {len(items)})\n")
            echo(format_output(items, fmt))

        except Exception as e:
            secho(f"Error: {e}", fg="red")
            sys.exit(1)

    @cli.command("list-statuses")
    @click.option("--format", "-f", "fmt", default="table",
                  type=click.Choice(["json", "table", "csv"]))
    def list_statuses(fmt: str) -> None:
        """List all statuses."""
        try:
            conn = get_connector()
            result = conn.metadata.get_statuses()

            items = []
            for status in result.get("results", []):
                items.append({
                    "id": status.get("id"),
                    "name": status.get("name"),
                    "description": (status.get("description") or "")[:40]
                })

            echo(format_output(items, fmt))

        except Exception as e:
            secho(f"Error: {e}", fg="red")
            sys.exit(1)

    # ==========================================================================
    # Bulk Operations
    # ==========================================================================

    @cli.command("bulk-update-status")
    @click.option("--input", "-i", "input_file", required=True,
                  help="CSV file with 'id' column")
    @click.option("--status-id", required=True, help="New status UUID")
    @click.option("--dry-run", is_flag=True, help="Preview without making changes")
    def bulk_update_status(input_file: str, status_id: str, dry_run: bool) -> None:
        """
        Update status for multiple assets from a CSV file.

        The input file should have an 'id' column with asset UUIDs.

        Examples:

          collibra-sdk bulk-update-status --input assets.csv --status-id "uuid"

          collibra-sdk bulk-update-status --input assets.csv --status-id "uuid" --dry-run
        """
        try:
            import pandas as pd

            conn = get_connector()
            df = pd.read_csv(input_file)

            if "id" not in df.columns:
                secho("Error: CSV must have an 'id' column", fg="red")
                sys.exit(1)

            asset_ids = df["id"].tolist()
            echo(f"Found {len(asset_ids)} assets to update")

            if dry_run:
                echo("Dry run - no changes made")
                return

            success = 0
            errors = 0

            with click.progressbar(asset_ids, label="Updating assets") as bar:
                for asset_id in bar:
                    try:
                        conn.asset.change_asset(asset_id=asset_id, status_id=status_id)
                        success += 1
                    except Exception:
                        errors += 1

            secho(f"Updated: {success}, Errors: {errors}", fg="green" if errors == 0 else "yellow")

        except ImportError:
            secho("Pandas required. Install with: pip install pandas", fg="red")
            sys.exit(1)
        except Exception as e:
            secho(f"Error: {e}", fg="red")
            sys.exit(1)


def main() -> None:
    """Entry point for the CLI."""
    if not CLICK_AVAILABLE:
        print("CLI requires 'click' package. Install with: pip install click")
        sys.exit(1)
    cli()


if __name__ == "__main__":
    main()



"""Tests for the Lineage Builder."""
import pytest
from collibra_connector.lineage import (
    LineageBuilder,
    LineageNode,
    LineageEdge,
    LineageCommitResult,
    LineageDirection,
    LineageRelationType,
)
from collibra_connector.testing import MockCollibraConnector


class TestLineageNode:
    """Tests for LineageNode."""

    def test_create_basic_node(self):
        node = LineageNode(name="Test Node", asset_type="Table")
        assert node.name == "Test Node"
        assert node.asset_type == "Table"
        assert node.asset_id is None

    def test_create_node_with_existing_id(self):
        node = LineageNode.from_id("existing-uuid", name="Existing")
        assert node.asset_id == "existing-uuid"

    def test_table_factory(self):
        node = LineageNode.table("orders", schema="public", database="postgres")
        assert node.name == "postgres.public.orders"
        assert node.asset_type == "Table"

    def test_column_factory(self):
        node = LineageNode.column("id", table="orders")
        assert node.name == "orders.id"
        assert node.asset_type == "Column"

    def test_pipeline_factory(self):
        node = LineageNode.pipeline("etl_job")
        assert node.name == "etl_job"
        assert node.asset_type == "Data Pipeline"

    def test_report_factory(self):
        node = LineageNode.report("Sales Report")
        assert node.asset_type == "Report"

    def test_dashboard_factory(self):
        node = LineageNode.dashboard("Analytics Dashboard")
        assert node.asset_type == "Dashboard"

    def test_node_with_attributes(self):
        node = LineageNode(
            name="Table",
            asset_type="Table",
            description="Test table",
            attributes={"Data Source": "S3"}
        )
        assert node.description == "Test table"
        assert node.attributes["Data Source"] == "S3"

    def test_node_equality(self):
        node1 = LineageNode(name="Table", asset_type="Table")
        node2 = LineageNode(name="Table", asset_type="Table")
        # Different nodes even with same name (different internal ID)
        assert node1 != node2

    def test_node_hashable(self):
        node = LineageNode(name="Table", asset_type="Table")
        node_set = {node}
        assert len(node_set) == 1


class TestLineageEdge:
    """Tests for LineageEdge."""

    def test_create_edge(self):
        source = LineageNode(name="Source", asset_type="Table")
        target = LineageNode(name="Target", asset_type="Table")
        edge = LineageEdge(source=source, target=target, relation_type="is source for")

        assert edge.source == source
        assert edge.target == target
        assert edge.relation_type == "is source for"


class TestLineageBuilder:
    """Tests for LineageBuilder."""

    def test_init(self):
        mock = MockCollibraConnector()
        builder = LineageBuilder(mock)
        assert builder.connector == mock

    def test_add_node(self):
        mock = MockCollibraConnector()
        builder = LineageBuilder(mock)
        node = LineageNode(name="Test", asset_type="Table")

        builder.add_node(node)

        assert len(builder.get_nodes()) == 1

    def test_add_edge(self):
        mock = MockCollibraConnector()
        builder = LineageBuilder(mock)

        source = LineageNode.table("source")
        target = LineageNode.table("target")

        builder.add_edge(source, target, "is source for")

        nodes = builder.get_nodes()
        edges = builder.get_edges()

        assert len(nodes) == 2
        assert len(edges) == 1
        assert edges[0].source == source
        assert edges[0].target == target

    def test_add_edges_batch(self):
        mock = MockCollibraConnector()
        builder = LineageBuilder(mock)

        a = LineageNode.table("a")
        b = LineageNode.table("b")
        c = LineageNode.table("c")

        builder.add_edges([
            (a, b, "is source for"),
            (b, c, "is source for"),
        ])

        assert len(builder.get_edges()) == 2

    def test_fluent_source_to(self):
        mock = MockCollibraConnector()
        builder = LineageBuilder(mock)

        source = LineageNode.table("source")
        target = LineageNode.table("target")

        builder.source(source).to(target)

        assert len(builder.get_edges()) == 1

    def test_fluent_source_through_to(self):
        mock = MockCollibraConnector()
        builder = LineageBuilder(mock)

        source = LineageNode.table("source")
        etl = LineageNode.pipeline("etl")
        target = LineageNode.table("target")

        builder.source(source).through(etl).to(target)

        edges = builder.get_edges()
        assert len(edges) == 2
        assert edges[0].source == source
        assert edges[0].target == etl
        assert edges[1].source == etl
        assert edges[1].target == target

    def test_chain(self):
        mock = MockCollibraConnector()
        builder = LineageBuilder(mock)

        a = LineageNode.table("a")
        b = LineageNode.pipeline("b")
        c = LineageNode.table("c")
        d = LineageNode.report("d")

        builder.chain(a, b, c, d)

        assert len(builder.get_edges()) == 3

    def test_fan_in(self):
        mock = MockCollibraConnector()
        builder = LineageBuilder(mock)

        sources = [LineageNode.table(f"src{i}") for i in range(3)]
        target = LineageNode.pipeline("etl")

        builder.fan_in(sources, target)

        edges = builder.get_edges()
        assert len(edges) == 3
        for edge in edges:
            assert edge.target == target

    def test_fan_out(self):
        mock = MockCollibraConnector()
        builder = LineageBuilder(mock)

        source = LineageNode.pipeline("etl")
        targets = [LineageNode.table(f"tgt{i}") for i in range(3)]

        builder.fan_out(source, targets)

        edges = builder.get_edges()
        assert len(edges) == 3
        for edge in edges:
            assert edge.source == source

    def test_clear(self):
        mock = MockCollibraConnector()
        builder = LineageBuilder(mock)

        builder.add_edge(
            LineageNode.table("a"),
            LineageNode.table("b")
        )
        builder.clear()

        assert len(builder.get_nodes()) == 0
        assert len(builder.get_edges()) == 0

    def test_to_dict(self):
        mock = MockCollibraConnector()
        builder = LineageBuilder(mock)

        source = LineageNode.table("source")
        target = LineageNode.table("target")
        builder.add_edge(source, target)

        data = builder.to_dict()

        assert "nodes" in data
        assert "edges" in data
        assert len(data["nodes"]) == 2
        assert len(data["edges"]) == 1

    def test_from_dict(self):
        mock = MockCollibraConnector()
        builder = LineageBuilder(mock)

        data = {
            "nodes": [
                {"id": "n1", "name": "Node 1", "asset_type": "Table"},
                {"id": "n2", "name": "Node 2", "asset_type": "Table"},
            ],
            "edges": [
                {"source": "n1", "target": "n2", "relation_type": "is source for"}
            ]
        }

        builder.from_dict(data)

        assert len(builder.get_nodes()) == 2
        assert len(builder.get_edges()) == 1

    def test_visualize(self):
        mock = MockCollibraConnector()
        builder = LineageBuilder(mock)

        source = LineageNode.table("source")
        target = LineageNode.table("target")
        builder.add_edge(source, target)

        viz = builder.visualize()

        assert "Lineage Graph" in viz
        assert "source" in viz
        assert "target" in viz

    def test_dry_run_commit(self):
        mock = MockCollibraConnector()
        builder = LineageBuilder(mock)

        source = LineageNode.table("source")
        target = LineageNode.table("target")
        builder.add_edge(source, target)

        result = builder.commit(domain_id="test-domain", dry_run=True)

        assert result.assets_created == 2
        assert result.relations_created == 1


class TestLineageCommitResult:
    """Tests for LineageCommitResult."""

    def test_default_values(self):
        result = LineageCommitResult()
        assert result.success is True
        assert result.assets_created == 0
        assert result.relations_created == 0
        assert len(result.errors) == 0

    def test_with_values(self):
        result = LineageCommitResult(
            success=True,
            assets_created=5,
            relations_created=3,
            assets={"n1": "a1", "n2": "a2"},
            relations=["r1", "r2"],
        )
        assert result.assets_created == 5
        assert len(result.assets) == 2
        assert len(result.relations) == 2


class TestLineageEnums:
    """Tests for lineage enums."""

    def test_direction_enum(self):
        assert LineageDirection.UPSTREAM == "upstream"
        assert LineageDirection.DOWNSTREAM == "downstream"
        assert LineageDirection.BIDIRECTIONAL == "bidirectional"

    def test_relation_type_enum(self):
        assert LineageRelationType.SOURCE_FOR == "is source for"
        assert LineageRelationType.TARGET_FOR == "is target for"
        assert LineageRelationType.TRANSFORMS == "transforms"



"""
Mocking Engine for Collibra Connector Testing.

This module provides utilities for testing code that uses the Collibra Connector
without making actual API calls. Perfect for unit tests and CI/CD pipelines.

Example:
    >>> from collibra_connector.testing import mock_collibra, MockCollibraConnector
    >>>
    >>> @mock_collibra
    ... def test_my_function():
    ...     conn = CollibraConnector(api="mock://", username="test", password="test")
    ...     asset = conn.asset.get_asset("any-uuid")
    ...     assert asset["name"] == "Mock Asset"

Or with custom responses:
    >>> from collibra_connector.testing import MockCollibraConnector
    >>>
    >>> def test_with_custom_data():
    ...     mock = MockCollibraConnector()
    ...     mock.asset.add_mock_asset({
    ...         "id": "custom-id",
    ...         "name": "Custom Asset",
    ...         "type": {"id": "type-1", "name": "Business Term"}
    ...     })
    ...
    ...     asset = mock.asset.get_asset("custom-id")
    ...     assert asset.name == "Custom Asset"
"""
from __future__ import annotations

import functools
import re
import uuid
from contextlib import contextmanager
from dataclasses import dataclass, field
from datetime import datetime
from typing import Any, Callable, Dict, Generator, List, Optional, TypeVar, Union
from unittest.mock import MagicMock, patch

from .models import (
    AssetModel,
    AssetList,
    DomainModel,
    DomainList,
    CommunityModel,
    CommunityList,
    UserModel,
    AttributeModel,
    AttributeList,
    RelationModel,
    RelationList,
    SearchResultModel,
    SearchResults,
    AssetProfileModel,
    ResourceReference,
    RelationsGrouped,
    RelationSummary,
    ResponsibilitySummary,
    PaginatedResponseModel,
    parse_asset,
    parse_assets,
)


T = TypeVar('T')
F = TypeVar('F', bound=Callable[..., Any])


def generate_uuid() -> str:
    """Generate a random UUID string."""
    return str(uuid.uuid4())


def generate_timestamp() -> int:
    """Generate current timestamp in milliseconds."""
    return int(datetime.now().timestamp() * 1000)


@dataclass
class MockResponse:
    """Mock HTTP response."""
    status_code: int = 200
    data: Dict[str, Any] = field(default_factory=dict)

    def json(self) -> Dict[str, Any]:
        return self.data

    @property
    def text(self) -> str:
        import json
        return json.dumps(self.data)


@dataclass
class MockDataStore:
    """In-memory data store for mock data."""
    assets: Dict[str, Dict[str, Any]] = field(default_factory=dict)
    domains: Dict[str, Dict[str, Any]] = field(default_factory=dict)
    communities: Dict[str, Dict[str, Any]] = field(default_factory=dict)
    users: Dict[str, Dict[str, Any]] = field(default_factory=dict)
    attributes: Dict[str, Dict[str, Any]] = field(default_factory=dict)
    relations: Dict[str, Dict[str, Any]] = field(default_factory=dict)
    asset_types: Dict[str, Dict[str, Any]] = field(default_factory=dict)
    statuses: Dict[str, Dict[str, Any]] = field(default_factory=dict)

    def clear(self) -> None:
        """Clear all data."""
        self.assets.clear()
        self.domains.clear()
        self.communities.clear()
        self.users.clear()
        self.attributes.clear()
        self.relations.clear()
        self.asset_types.clear()
        self.statuses.clear()


# Global mock data store
_mock_store = MockDataStore()


def _create_default_asset(
    asset_id: Optional[str] = None,
    name: str = "Mock Asset",
    **kwargs: Any
) -> Dict[str, Any]:
    """Create a default mock asset."""
    aid = asset_id or generate_uuid()
    return {
        "id": aid,
        "resourceType": "Asset",
        "name": name,
        "displayName": kwargs.get("display_name", name),
        "description": kwargs.get("description"),
        "type": kwargs.get("type", {
            "id": generate_uuid(),
            "resourceType": "AssetType",
            "name": "Business Term"
        }),
        "status": kwargs.get("status", {
            "id": generate_uuid(),
            "resourceType": "Status",
            "name": "Approved"
        }),
        "domain": kwargs.get("domain", {
            "id": generate_uuid(),
            "resourceType": "Domain",
            "name": "Mock Domain"
        }),
        "avgRating": kwargs.get("avg_rating", 0.0),
        "ratingsCount": kwargs.get("ratings_count", 0),
        "createdOn": kwargs.get("created_on", generate_timestamp()),
        "lastModifiedOn": kwargs.get("last_modified_on", generate_timestamp()),
        "excludedFromAutoHyperlinking": kwargs.get("excluded_from_auto_hyperlinking", False)
    }


def _create_default_domain(
    domain_id: Optional[str] = None,
    name: str = "Mock Domain",
    **kwargs: Any
) -> Dict[str, Any]:
    """Create a default mock domain."""
    did = domain_id or generate_uuid()
    return {
        "id": did,
        "resourceType": "Domain",
        "name": name,
        "description": kwargs.get("description"),
        "type": kwargs.get("type", {
            "id": generate_uuid(),
            "resourceType": "DomainType",
            "name": "Business Glossary"
        }),
        "community": kwargs.get("community", {
            "id": generate_uuid(),
            "resourceType": "Community",
            "name": "Mock Community"
        }),
        "createdOn": generate_timestamp(),
        "lastModifiedOn": generate_timestamp()
    }


def _create_default_community(
    community_id: Optional[str] = None,
    name: str = "Mock Community",
    **kwargs: Any
) -> Dict[str, Any]:
    """Create a default mock community."""
    cid = community_id or generate_uuid()
    return {
        "id": cid,
        "resourceType": "Community",
        "name": name,
        "description": kwargs.get("description"),
        "parent": kwargs.get("parent"),
        "createdOn": generate_timestamp(),
        "lastModifiedOn": generate_timestamp()
    }


class MockAssetAPI:
    """Mock Asset API with typed returns."""

    def __init__(self, store: MockDataStore) -> None:
        self._store = store

    def add_mock_asset(self, data: Dict[str, Any]) -> str:
        """Add a mock asset to the store."""
        asset_id = data.get("id", generate_uuid())
        self._store.assets[asset_id] = {**_create_default_asset(asset_id), **data}
        return asset_id

    def get_asset(self, asset_id: str) -> AssetModel:
        """Get an asset by ID."""
        if asset_id in self._store.assets:
            return parse_asset(self._store.assets[asset_id])

        # Return default mock asset
        return parse_asset(_create_default_asset(asset_id))

    def find_assets(
        self,
        community_id: Optional[str] = None,
        domain_id: Optional[str] = None,
        asset_type_ids: Optional[List[str]] = None,
        limit: int = 100,
        offset: int = 0,
        **kwargs: Any
    ) -> AssetList:
        """Find assets with filters."""
        results = list(self._store.assets.values())

        # Apply filters
        if domain_id:
            results = [a for a in results if a.get("domain", {}).get("id") == domain_id]
        if community_id:
            results = [a for a in results if a.get("community", {}).get("id") == community_id]
        if asset_type_ids:
            results = [a for a in results if a.get("type", {}).get("id") in asset_type_ids]

        # If no assets in store, return mock data
        if not results:
            results = [_create_default_asset() for _ in range(min(3, limit))]

        # Apply pagination
        total = len(results)
        results = results[offset:offset + limit]

        return parse_assets({
            "results": results,
            "total": total,
            "offset": offset,
            "limit": limit
        })

    def add_asset(
        self,
        name: str,
        domain_id: str,
        type_id: Optional[str] = None,
        status_id: Optional[str] = None,
        display_name: Optional[str] = None,
        **kwargs: Any
    ) -> AssetModel:
        """Create a new mock asset."""
        asset_id = generate_uuid()
        asset = _create_default_asset(
            asset_id=asset_id,
            name=name,
            display_name=display_name or name
        )
        if type_id:
            asset["type"]["id"] = type_id
        if status_id:
            asset["status"]["id"] = status_id
        asset["domain"]["id"] = domain_id

        self._store.assets[asset_id] = asset
        return parse_asset(asset)

    def change_asset(
        self,
        asset_id: str,
        **kwargs: Any
    ) -> AssetModel:
        """Update a mock asset."""
        if asset_id in self._store.assets:
            asset = self._store.assets[asset_id]
            if "name" in kwargs and kwargs["name"]:
                asset["name"] = kwargs["name"]
            if "display_name" in kwargs and kwargs["display_name"]:
                asset["displayName"] = kwargs["display_name"]
            if "status_id" in kwargs and kwargs["status_id"]:
                asset["status"]["id"] = kwargs["status_id"]
            return parse_asset(asset)

        return parse_asset(_create_default_asset(asset_id))

    def remove_asset(self, asset_id: str) -> None:
        """Remove a mock asset."""
        self._store.assets.pop(asset_id, None)

    def get_full_profile(
        self,
        asset_id: str,
        include_attributes: bool = True,
        include_relations: bool = True,
        include_responsibilities: bool = True,
        **kwargs: Any
    ) -> AssetProfileModel:
        """Get full profile for an asset."""
        asset = self.get_asset(asset_id)

        return AssetProfileModel(
            asset=asset,
            attributes={"Description": "Mock description", "Definition": "Mock definition"},
            relations=RelationsGrouped(
                outgoing={"contains": [RelationSummary(id=generate_uuid(), target_id=generate_uuid(), target_name="Related Asset")]},
                incoming={},
                outgoing_count=1,
                incoming_count=0
            ),
            responsibilities=[ResponsibilitySummary(role="Data Steward", owner="Mock User", owner_id=generate_uuid())]
        )


class MockAttributeAPI:
    """Mock Attribute API."""

    def __init__(self, store: MockDataStore) -> None:
        self._store = store

    def get_attributes(
        self,
        asset_id: str,
        limit: int = 100,
        offset: int = 0,
        **kwargs: Any
    ) -> AttributeList:
        """Get attributes for an asset."""
        # Return mock attributes
        attrs = [
            {
                "id": generate_uuid(),
                "resourceType": "Attribute",
                "type": {"id": generate_uuid(), "name": "Description"},
                "value": "Mock description",
                "asset": {"id": asset_id}
            },
            {
                "id": generate_uuid(),
                "resourceType": "Attribute",
                "type": {"id": generate_uuid(), "name": "Definition"},
                "value": "Mock definition",
                "asset": {"id": asset_id}
            }
        ]
        return PaginatedResponseModel[AttributeModel](
            results=[AttributeModel.model_validate(a) for a in attrs],
            total=len(attrs),
            offset=offset,
            limit=limit
        )

    def get_attributes_as_dict(self, asset_id: str) -> Dict[str, Any]:
        """Get attributes as dict."""
        return {
            "Description": "Mock description",
            "Definition": "Mock definition"
        }

    def add_attribute(
        self,
        asset_id: str,
        type_id: str,
        value: Any
    ) -> AttributeModel:
        """Add an attribute."""
        attr = {
            "id": generate_uuid(),
            "resourceType": "Attribute",
            "type": {"id": type_id, "name": "Custom"},
            "value": value,
            "asset": {"id": asset_id}
        }
        return AttributeModel.model_validate(attr)


class MockDomainAPI:
    """Mock Domain API."""

    def __init__(self, store: MockDataStore) -> None:
        self._store = store

    def add_mock_domain(self, data: Dict[str, Any]) -> str:
        """Add a mock domain to the store."""
        domain_id = data.get("id", generate_uuid())
        self._store.domains[domain_id] = {**_create_default_domain(domain_id), **data}
        return domain_id

    def get_domain(self, domain_id: str) -> DomainModel:
        """Get a domain by ID."""
        if domain_id in self._store.domains:
            return DomainModel.model_validate(self._store.domains[domain_id])
        return DomainModel.model_validate(_create_default_domain(domain_id))

    def find_domains(
        self,
        community_id: Optional[str] = None,
        limit: int = 100,
        offset: int = 0,
        **kwargs: Any
    ) -> DomainList:
        """Find domains."""
        results = list(self._store.domains.values())
        if community_id:
            results = [d for d in results if d.get("community", {}).get("id") == community_id]

        if not results:
            results = [_create_default_domain() for _ in range(min(2, limit))]

        return PaginatedResponseModel[DomainModel](
            results=[DomainModel.model_validate(d) for d in results[offset:offset + limit]],
            total=len(results),
            offset=offset,
            limit=limit
        )


class MockCommunityAPI:
    """Mock Community API."""

    def __init__(self, store: MockDataStore) -> None:
        self._store = store

    def get_community(self, community_id: str) -> CommunityModel:
        """Get a community by ID."""
        if community_id in self._store.communities:
            return CommunityModel.model_validate(self._store.communities[community_id])
        return CommunityModel.model_validate(_create_default_community(community_id))

    def find_communities(
        self,
        limit: int = 100,
        offset: int = 0,
        **kwargs: Any
    ) -> CommunityList:
        """Find communities."""
        results = list(self._store.communities.values())
        if not results:
            results = [_create_default_community() for _ in range(min(2, limit))]

        return PaginatedResponseModel[CommunityModel](
            results=[CommunityModel.model_validate(c) for c in results[offset:offset + limit]],
            total=len(results),
            offset=offset,
            limit=limit
        )


class MockRelationAPI:
    """Mock Relation API."""

    def __init__(self, store: MockDataStore) -> None:
        self._store = store

    def add_relation(
        self,
        source_id: str,
        target_id: str,
        type_id: str,
        **kwargs: Any
    ) -> RelationModel:
        """Create a relation."""
        rel = {
            "id": generate_uuid(),
            "resourceType": "Relation",
            "type": {"id": type_id, "name": "is source for"},
            "source": {"id": source_id, "name": "Source Asset"},
            "target": {"id": target_id, "name": "Target Asset"}
        }
        self._store.relations[rel["id"]] = rel
        return RelationModel.model_validate(rel)

    def find_relations(
        self,
        source_id: Optional[str] = None,
        target_id: Optional[str] = None,
        limit: int = 100,
        offset: int = 0,
        **kwargs: Any
    ) -> RelationList:
        """Find relations."""
        results = list(self._store.relations.values())
        if source_id:
            results = [r for r in results if r.get("source", {}).get("id") == source_id]
        if target_id:
            results = [r for r in results if r.get("target", {}).get("id") == target_id]

        return PaginatedResponseModel[RelationModel](
            results=[RelationModel.model_validate(r) for r in results[offset:offset + limit]],
            total=len(results),
            offset=offset,
            limit=limit
        )

    def get_asset_relations(
        self,
        asset_id: str,
        **kwargs: Any
    ) -> Dict[str, Any]:
        """Get relations for an asset."""
        return {
            "outgoing": {},
            "incoming": {},
            "outgoing_count": 0,
            "incoming_count": 0
        }


class MockSearchAPI:
    """Mock Search API."""

    def __init__(self, store: MockDataStore) -> None:
        self._store = store

    def find(
        self,
        query: str,
        limit: int = 10,
        offset: int = 0,
        **kwargs: Any
    ) -> SearchResults:
        """Search for assets."""
        # Simple matching on asset names
        results = []
        pattern = query.replace("*", ".*")

        for asset in self._store.assets.values():
            if re.search(pattern, asset.get("name", ""), re.IGNORECASE):
                results.append({
                    "resource": {
                        "id": asset["id"],
                        "resourceType": "Asset",
                        "name": asset["name"],
                        "displayName": asset.get("displayName")
                    },
                    "score": 1.0
                })

        # Add default results if empty
        if not results:
            results = [{
                "resource": {
                    "id": generate_uuid(),
                    "resourceType": "Asset",
                    "name": f"Result for: {query}",
                    "displayName": f"Result for: {query}"
                },
                "score": 0.9
            }]

        return PaginatedResponseModel[SearchResultModel](
            results=[SearchResultModel.model_validate(r) for r in results[offset:offset + limit]],
            total=len(results),
            offset=offset,
            limit=limit
        )

    def find_assets(self, query: str, **kwargs: Any) -> SearchResults:
        """Search specifically for assets."""
        return self.find(query, **kwargs)


class MockMetadataAPI:
    """Mock Metadata API."""

    def __init__(self, store: MockDataStore) -> None:
        self._store = store

    def get_asset_types(self, name: Optional[str] = None, limit: int = 100, **kwargs: Any) -> Dict[str, Any]:
        """Get asset types."""
        types = [
            {"id": generate_uuid(), "name": "Business Term", "publicId": "BusinessTerm"},
            {"id": generate_uuid(), "name": "Table", "publicId": "Table"},
            {"id": generate_uuid(), "name": "Column", "publicId": "Column"},
            {"id": generate_uuid(), "name": "Data Pipeline", "publicId": "DataPipeline"}
        ]
        if name:
            types = [t for t in types if name.lower() in t["name"].lower()]
        return {"results": types[:limit], "total": len(types)}

    def get_statuses(self, limit: int = 100, **kwargs: Any) -> Dict[str, Any]:
        """Get statuses."""
        statuses = [
            {"id": generate_uuid(), "name": "Approved"},
            {"id": generate_uuid(), "name": "Pending"},
            {"id": generate_uuid(), "name": "Rejected"},
            {"id": generate_uuid(), "name": "Draft"}
        ]
        return {"results": statuses[:limit], "total": len(statuses)}

    def get_relation_types(self, role: Optional[str] = None, limit: int = 100, **kwargs: Any) -> Dict[str, Any]:
        """Get relation types."""
        types = [
            {"id": generate_uuid(), "name": "contains", "role": "contains", "coRole": "is part of"},
            {"id": generate_uuid(), "name": "is source for", "role": "is source for", "coRole": "is target for"}
        ]
        if role:
            types = [t for t in types if role.lower() in (t.get("role") or "").lower()]
        return {"results": types[:limit], "total": len(types)}


class MockResponsibilityAPI:
    """Mock Responsibility API."""

    def __init__(self, store: MockDataStore) -> None:
        self._store = store

    def get_asset_responsibilities(self, asset_id: str, **kwargs: Any) -> List[Dict[str, Any]]:
        """Get responsibilities for an asset."""
        return [
            {"role": "Data Steward", "owner": "Mock User", "owner_id": generate_uuid()},
            {"role": "Technical Owner", "owner": "Tech User", "owner_id": generate_uuid()}
        ]


class MockCollibraConnector:
    """
    Mock Collibra Connector for testing.

    This class provides the same interface as CollibraConnector
    but returns mock data instead of making API calls.

    Example:
        >>> mock = MockCollibraConnector()
        >>>
        >>> # Add custom test data
        >>> mock.asset.add_mock_asset({
        ...     "id": "test-id",
        ...     "name": "Test Asset",
        ...     "type": {"id": "type-1", "name": "Business Term"}
        ... })
        >>>
        >>> # Use like regular connector
        >>> asset = mock.asset.get_asset("test-id")
        >>> assert asset.name == "Test Asset"
        >>>
        >>> # Find assets
        >>> results = mock.asset.find_assets(limit=10)
        >>> for asset in results:
        ...     print(asset.name)
    """

    def __init__(
        self,
        api: str = "mock://collibra.test",
        username: str = "test",
        password: str = "test",
        **kwargs: Any
    ) -> None:
        """Initialize mock connector."""
        self._api = api
        self._base_url = api
        self._store = MockDataStore()

        # Initialize mock APIs
        self.asset = MockAssetAPI(self._store)
        self.attribute = MockAttributeAPI(self._store)
        self.domain = MockDomainAPI(self._store)
        self.community = MockCommunityAPI(self._store)
        self.relation = MockRelationAPI(self._store)
        self.search = MockSearchAPI(self._store)
        self.metadata = MockMetadataAPI(self._store)
        self.responsibility = MockResponsibilityAPI(self._store)

    @property
    def api(self) -> str:
        return self._api

    @property
    def base_url(self) -> str:
        return self._base_url

    def test_connection(self) -> bool:
        """Always returns True for mock."""
        return True

    def get_version(self) -> str:
        """Get mock version."""
        return "1.1.0-mock"

    def clear_all_data(self) -> None:
        """Clear all mock data."""
        self._store.clear()

    def __enter__(self) -> "MockCollibraConnector":
        return self

    def __exit__(self, *args: Any) -> None:
        pass


@contextmanager
def mock_collibra_context(
    custom_data: Optional[Dict[str, Any]] = None
) -> Generator[MockCollibraConnector, None, None]:
    """
    Context manager for mocking Collibra in tests.

    Args:
        custom_data: Optional dict with custom mock data.

    Yields:
        MockCollibraConnector instance.

    Example:
        >>> with mock_collibra_context() as mock_conn:
        ...     mock_conn.asset.add_mock_asset({"name": "Test"})
        ...     result = mock_conn.asset.find_assets()
        ...     assert len(result) > 0
    """
    mock = MockCollibraConnector()

    if custom_data:
        for asset in custom_data.get("assets", []):
            mock.asset.add_mock_asset(asset)
        for domain in custom_data.get("domains", []):
            mock.domain.add_mock_domain(domain)

    yield mock


def mock_collibra(func: Optional[F] = None, **kwargs: Any) -> Union[F, Callable[[F], F]]:
    """
    Decorator for mocking Collibra in tests.

    Can be used with or without arguments.

    Examples:
        >>> @mock_collibra
        ... def test_something():
        ...     conn = CollibraConnector(api="mock://", username="x", password="y")
        ...     # This will use mock data
        ...     asset = conn.asset.get_asset("any-id")
        >>>
        >>> @mock_collibra(custom_assets=[{"name": "Custom"}])
        ... def test_with_data():
        ...     pass
    """
    def decorator(f: F) -> F:
        @functools.wraps(f)
        def wrapper(*args: Any, **kw: Any) -> Any:
            # Patch CollibraConnector to return MockCollibraConnector
            with patch('collibra_connector.CollibraConnector', MockCollibraConnector):
                with patch('collibra_connector.connector.CollibraConnector', MockCollibraConnector):
                    return f(*args, **kw)
        return wrapper  # type: ignore

    if func is not None:
        return decorator(func)
    return decorator


class CollibraTestCase:
    """
    Base test case class for Collibra tests.

    Provides setUp/tearDown with automatic mocking.

    Example:
        >>> class TestMyFeature(CollibraTestCase):
        ...     def test_asset_creation(self):
        ...         asset = self.connector.asset.add_asset(
        ...             name="Test",
        ...             domain_id="domain-uuid"
        ...         )
        ...         assert asset.name == "Test"
    """

    def setUp(self) -> None:
        """Set up mock connector."""
        self.connector = MockCollibraConnector()

    def tearDown(self) -> None:
        """Clean up mock data."""
        self.connector.clear_all_data()

    def add_test_asset(self, **kwargs: Any) -> str:
        """Helper to add a test asset."""
        data = {
            "name": kwargs.get("name", "Test Asset"),
            **kwargs
        }
        return self.connector.asset.add_mock_asset(data)

    def add_test_domain(self, **kwargs: Any) -> str:
        """Helper to add a test domain."""
        data = {
            "name": kwargs.get("name", "Test Domain"),
            **kwargs
        }
        return self.connector.domain.add_mock_domain(data)



import requests
from typing import Optional, Dict, Any
from .Base import BaseAPI


class OutputModule(BaseAPI):
    """
    Output Module API endpoints for Collibra DGC.
    """
    def __init__(self, connector):
        super().__init__(connector)
        self.__base_api = connector.api + "/outputModule"

    def export_json(
        self,
        body: str,
        validation_enabled: bool = False
    ) -> Dict[Any, Any]:
        """
        Exports results in JSON format, returns the results immediately.

        Performs an Output Module query and exports the returns results immediately in JSON format.

        Please note that the ViewConfig/TableViewConfig's syntax validation is not executed by default.
        DGC admin console settings may impact the execution of the query (especially in terms of timeout
        and a limit on the number of results).

        Args:
            view_config (str): The JSON/YAML representation of ViewConfig/TableViewConfig
                             that describes the query to be performed.
            validation_enabled (bool): Determines if the ViewConfig's syntax should be validated
                                     (True) or not (False). Default value is False for backward
                                     compatibility reasons but it is strongly advised to always
                                     enable this validation.

        Returns:
            Dict[Any, Any]: The exported results in JSON format.

        Raises:
            requests.exceptions.RequestException: If the API request fails.
        """
        endpoint = f"{self.__base_api}/export/json"

        headers = {
            'Content-Type': 'application/json'
        }

        return self._post(url=endpoint, data=body, headers=headers).json()



"""Tests for Pydantic Models."""
import pytest
from collibra_connector.models import (
    AssetModel,
    DomainModel,
    CommunityModel,
    UserModel,
    AttributeModel,
    RelationModel,
    ResourceReference,
    PaginatedResponseModel,
    AssetProfileModel,
    RelationsGrouped,
    ResponsibilitySummary,
    SearchResultModel,
    parse_asset,
    parse_assets,
    parse_domain,
    parse_community,
    parse_user,
)


class TestResourceReference:
    """Tests for ResourceReference model."""

    def test_create_from_dict(self):
        data = {"id": "123", "resourceType": "Asset", "name": "Test"}
        ref = ResourceReference.model_validate(data)
        assert ref.id == "123"
        assert ref.resource_type == "Asset"
        assert ref.name == "Test"

    def test_string_representation(self):
        ref = ResourceReference(id="123", name="Test Asset")
        assert str(ref) == "Test Asset"

    def test_string_representation_no_name(self):
        ref = ResourceReference(id="123")
        assert str(ref) == "123"


class TestAssetModel:
    """Tests for AssetModel."""

    def test_create_from_dict(self):
        data = {
            "id": "asset-123",
            "resourceType": "Asset",
            "name": "Test Asset",
            "displayName": "Test Display",
            "type": {"id": "type-1", "resourceType": "AssetType", "name": "Business Term"},
            "status": {"id": "status-1", "resourceType": "Status", "name": "Approved"},
            "domain": {"id": "domain-1", "resourceType": "Domain", "name": "Glossary"},
            "avgRating": 4.5,
            "ratingsCount": 10,
            "createdOn": 1704067200000,
            "lastModifiedOn": 1704153600000,
        }
        asset = AssetModel.model_validate(data)

        assert asset.id == "asset-123"
        assert asset.name == "Test Asset"
        assert asset.display_name == "Test Display"
        assert asset.type.name == "Business Term"
        assert asset.status.name == "Approved"
        assert asset.domain.name == "Glossary"
        assert asset.avg_rating == 4.5
        assert asset.ratings_count == 10

    def test_effective_name(self):
        asset = AssetModel(
            id="1",
            name="Name",
            display_name="Display Name",
            type=ResourceReference(id="t1", name="Type"),
            status=ResourceReference(id="s1", name="Status"),
            domain=ResourceReference(id="d1", name="Domain"),
        )
        assert asset.effective_name == "Display Name"

    def test_effective_name_no_display(self):
        asset = AssetModel(
            id="1",
            name="Name",
            type=ResourceReference(id="t1", name="Type"),
            status=ResourceReference(id="s1", name="Status"),
            domain=ResourceReference(id="d1", name="Domain"),
        )
        assert asset.effective_name == "Name"

    def test_convenience_properties(self):
        asset = AssetModel(
            id="1",
            name="Name",
            type=ResourceReference(id="t1", name="Business Term"),
            status=ResourceReference(id="s1", name="Approved"),
            domain=ResourceReference(id="d1", name="Glossary"),
        )
        assert asset.type_name == "Business Term"
        assert asset.status_name == "Approved"
        assert asset.domain_name == "Glossary"

    def test_has_rating(self):
        asset = AssetModel(
            id="1",
            name="Name",
            type=ResourceReference(id="t1", name="Type"),
            status=ResourceReference(id="s1", name="Status"),
            domain=ResourceReference(id="d1", name="Domain"),
            ratings_count=5,
        )
        assert asset.has_rating is True

        asset2 = AssetModel(
            id="1",
            name="Name",
            type=ResourceReference(id="t1", name="Type"),
            status=ResourceReference(id="s1", name="Status"),
            domain=ResourceReference(id="d1", name="Domain"),
            ratings_count=0,
        )
        assert asset2.has_rating is False

    def test_to_dict(self):
        asset = AssetModel(
            id="1",
            name="Name",
            type=ResourceReference(id="t1", name="Type"),
            status=ResourceReference(id="s1", name="Status"),
            domain=ResourceReference(id="d1", name="Domain"),
        )
        d = asset.to_dict()
        assert d["id"] == "1"
        assert d["name"] == "Name"

    def test_created_datetime(self):
        asset = AssetModel(
            id="1",
            name="Name",
            type=ResourceReference(id="t1", name="Type"),
            status=ResourceReference(id="s1", name="Status"),
            domain=ResourceReference(id="d1", name="Domain"),
            created_on=1704067200000,
        )
        dt = asset.created_datetime
        assert dt is not None
        assert dt.year == 2024


class TestUserModel:
    """Tests for UserModel."""

    def test_full_name(self):
        user = UserModel(
            id="u1",
            name="jdoe",
            first_name="John",
            last_name="Doe",
        )
        assert user.full_name == "John Doe"

    def test_full_name_partial(self):
        user = UserModel(id="u1", name="jdoe", first_name="John")
        assert user.full_name == "John"

    def test_is_active(self):
        user = UserModel(id="u1", name="jdoe", activated=True, enabled=True)
        assert user.is_active is True

        user2 = UserModel(id="u1", name="jdoe", activated=True, enabled=False)
        assert user2.is_active is False


class TestPaginatedResponseModel:
    """Tests for PaginatedResponseModel."""

    def test_iteration(self):
        assets = [
            AssetModel(
                id=f"a{i}",
                name=f"Asset {i}",
                type=ResourceReference(id="t1", name="Type"),
                status=ResourceReference(id="s1", name="Status"),
                domain=ResourceReference(id="d1", name="Domain"),
            )
            for i in range(3)
        ]
        paginated = PaginatedResponseModel[AssetModel](
            results=assets, total=10, offset=0, limit=3
        )

        names = [a.name for a in paginated]
        assert names == ["Asset 0", "Asset 1", "Asset 2"]

    def test_has_more(self):
        paginated = PaginatedResponseModel[AssetModel](
            results=[], total=10, offset=0, limit=3
        )
        assert paginated.has_more is True

        paginated2 = PaginatedResponseModel[AssetModel](
            results=[], total=3, offset=0, limit=3
        )
        assert paginated2.has_more is False

    def test_next_offset(self):
        assets = [
            AssetModel(
                id="a1",
                name="Asset",
                type=ResourceReference(id="t1", name="Type"),
                status=ResourceReference(id="s1", name="Status"),
                domain=ResourceReference(id="d1", name="Domain"),
            )
        ] * 3
        paginated = PaginatedResponseModel[AssetModel](
            results=assets, total=10, offset=0, limit=3
        )
        assert paginated.next_offset == 3

    def test_page_count(self):
        paginated = PaginatedResponseModel[AssetModel](
            results=[], total=25, offset=0, limit=10
        )
        assert paginated.page_count == 3

    def test_current_page(self):
        paginated = PaginatedResponseModel[AssetModel](
            results=[], total=25, offset=10, limit=10
        )
        assert paginated.current_page == 2

    def test_len(self):
        assets = [
            AssetModel(
                id="a1",
                name="Asset",
                type=ResourceReference(id="t1", name="Type"),
                status=ResourceReference(id="s1", name="Status"),
                domain=ResourceReference(id="d1", name="Domain"),
            )
        ] * 5
        paginated = PaginatedResponseModel[AssetModel](
            results=assets, total=100, offset=0, limit=5
        )
        assert len(paginated) == 5

    def test_bool(self):
        paginated = PaginatedResponseModel[AssetModel](
            results=[], total=0, offset=0, limit=10
        )
        assert bool(paginated) is False

        assets = [
            AssetModel(
                id="a1",
                name="Asset",
                type=ResourceReference(id="t1", name="Type"),
                status=ResourceReference(id="s1", name="Status"),
                domain=ResourceReference(id="d1", name="Domain"),
            )
        ]
        paginated2 = PaginatedResponseModel[AssetModel](
            results=assets, total=1, offset=0, limit=10
        )
        assert bool(paginated2) is True


class TestParseAsset:
    """Tests for parse_asset function."""

    def test_parse_single_asset(self):
        data = {
            "id": "asset-123",
            "name": "Test Asset",
            "type": {"id": "t1", "name": "Business Term"},
            "status": {"id": "s1", "name": "Approved"},
            "domain": {"id": "d1", "name": "Glossary"},
        }
        asset = parse_asset(data)
        assert isinstance(asset, AssetModel)
        assert asset.name == "Test Asset"

    def test_parse_assets_list(self):
        data = {
            "results": [
                {
                    "id": f"a{i}",
                    "name": f"Asset {i}",
                    "type": {"id": "t1", "name": "Type"},
                    "status": {"id": "s1", "name": "Status"},
                    "domain": {"id": "d1", "name": "Domain"},
                }
                for i in range(3)
            ],
            "total": 10,
            "offset": 0,
            "limit": 3,
        }
        result = parse_assets(data)
        assert isinstance(result, PaginatedResponseModel)
        assert len(result) == 3
        assert result.total == 10


class TestAssetProfileModel:
    """Tests for AssetProfileModel."""

    def test_create_profile(self):
        asset = AssetModel(
            id="1",
            name="Test Asset",
            type=ResourceReference(id="t1", name="Business Term"),
            status=ResourceReference(id="s1", name="Approved"),
            domain=ResourceReference(id="d1", name="Glossary"),
        )
        profile = AssetProfileModel(
            asset=asset,
            attributes={"Description": "Test description"},
            relations=RelationsGrouped(),
            responsibilities=[
                ResponsibilitySummary(role="Data Steward", owner="John Doe")
            ],
        )

        assert profile.description == "Test description"
        assert profile.data_steward == "John Doe"

    def test_data_steward_not_found(self):
        asset = AssetModel(
            id="1",
            name="Test Asset",
            type=ResourceReference(id="t1", name="Type"),
            status=ResourceReference(id="s1", name="Status"),
            domain=ResourceReference(id="d1", name="Domain"),
        )
        profile = AssetProfileModel(
            asset=asset,
            attributes={},
            relations=RelationsGrouped(),
            responsibilities=[
                ResponsibilitySummary(role="Technical Owner", owner="Jane Doe")
            ],
        )

        assert profile.data_steward is None


class TestSearchResultModel:
    """Tests for SearchResultModel."""

    def test_search_result_model_parsing(self):
        """Test parsing JSON into SearchResultModel."""
        data = {
            "resource": {
                "id": "uuid-s",
                "resourceType": "Asset",
                "name": "Search Result",
                "displayName": "Display Search"
            },
            "score": 1.5,
            "highlights": [{"field": "name", "values": ["Search"]}]
        }

        result = SearchResultModel.model_validate(data)
        assert result.resource.id == "uuid-s"
        assert result.resource.name == "Search Result"
        assert result.score == 1.5
        assert result.id == "uuid-s"
        assert result.name == "Display Search"


class TestDomainModel:
    """Tests for DomainModel."""

    def test_domain_model_parsing(self):
        """Test parsing JSON into DomainModel."""
        data = {
            "id": "dom-1",
            "resourceType": "Domain",
            "name": "Test Domain",
            "type": {"id": "type-d", "resourceType": "DomainType", "name": "Glossary"},
            "community": {"id": "comm-1", "resourceType": "Community", "name": "Comm 1"},
            "excludedFromAutoHyperlinking": True
        }

        domain = DomainModel.model_validate(data)

        assert domain.name == "Test Domain"
        assert domain.community.name == "Comm 1"
        assert domain.excluded_from_auto_hyperlinking is True


class TestCommunityModel:
    """Tests for CommunityModel."""

    def test_is_root(self):
        comm = CommunityModel(id="c1", name="Root Community")
        assert comm.is_root() is True

        comm2 = CommunityModel(
            id="c2",
            name="Child Community",
            parent=ResourceReference(id="c1", name="Root")
        )
        assert comm2.is_root() is False
